/*****************************************************************************/
/*                                                                           */
/*  FileName:  ssyse002.cpp                                                  */
/*                                                                           */
/*  Type:  C++ Code Sample for AIX, NT and OS/2                              */
/*                                                                           */
/*  Called by:  ssyse002.mac                                                 */
/*                                                                           */
/*                    ------------------------------------                   */
/*   Licensed Materials - Property of IBM                                    */
/*                                                                           */
/*   (C) COPYRIGHT International Business Machines Corp. 1996                */
/*    All Rights Reserved.                                                   */
/*                                                                           */
/*   US Government Users Restricted Rights - Use, duplication or             */
/*   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.       */
/*                                                                           */
/*                    ------------------------------------                   */
/*                                                                           */
/*               NOTICE TO USERS OF THE SOURCE CODE EXAMPLES                 */
/*                                                                           */
/*  INTERNATIONAL BUSINESS MACHINES CORPORATION PROVIDES THE SOURCE CODE     */
/*  EXAMPLES, BOTH INDIVIDUALLY AND AS ONE OR MORE GROUPS, "AS IS" WITHOUT   */
/*  WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT    */
/*  LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A   */
/*  PARTICULAR PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE   */
/*  OF THE SOURCE CODE EXAMPLES, BOTH INDIVIDUALLY AND AS ONE OR MORE GROUPS,*/
/*  IS WITH YOU.  SHOULD ANY PART OF THE SOURCE CODE EXAMPLES PROVE          */
/*  DEFECTIVE, YOU (AND NOT IBM) ASSUME THE ENTIRE COST IF ALL NECESSARY     */
/*  SERVICING, REPAIR OR CORRECTION.                                         */
/*****************************************************************************/
 
/*****************************************************************************/
/*   GLOBAL includes                                                         */
/*****************************************************************************/
#include <io.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
 
 
/*****************************************************************************/
/*  Function:  Main                                                          */
/*                                                                           */
/*  Description: Display the input arguments and parameters and use them     */
/*               to create a table. Assign the value of parameter1 to        */
/*               parameter2. Return the table and parameter2 to the caller.  */
/*                                                                           */
/*  Inputs:   arg[1], arg[2], arg[3], parameter1, parameter2, DTWPIPE        */
/*                                                                           */
/*  Outputs:  parameter2, table                                              */
/*                                                                           */
/*  Returns:  N/A                                                            */
/*                                                                           */
/*****************************************************************************/
int main(int argc, char *argv[]) {
 
/*****************************************************************************/
/*                                                                           */
/*   Define the local variables                                              */
/*                                                                           */
/*****************************************************************************/
 int pipeHandle,
     rc=0;

 /* for simplicity lets create an 8KB buffer  */ 
 char   *outmsg = (char*) malloc(sizeof(char)*8192);

/*****************************************************************************/
/*                                                                           */
/*   Check to make sure that exactly four arguments have been passed.        */
/*                                                                           */
/*****************************************************************************/
   if (argc != 4) {
      printf ("\nUSAGE: requires exactly four arguments.\n\n");
      return 1;
   }  /* endif */
 
   printf("<em>... ssyse002.exe begins ...</em>\n\n");
 
/*****************************************************************************/
/*                                                                           */
/*   Display the input arguments.                                            */
/*                                                                           */
/*****************************************************************************/
   printf("Input arguments\n");
   printf("Argument 1 is: %s\n", argv[1]);
   printf("Argument 2 is: %s\n", argv[2]);
   printf("Argument 3 is: %s\n\n\n", argv[3]);

/*****************************************************************************/
/*                                                                           */
/*   Get the parameters from their environment variables and display them.   */
/*                                                                           */
/*****************************************************************************/
   char *envparameter1 = getenv("PARAMETER1");
   char *envparameter2 = getenv("PARAMETER2");
   char *pipeName      = getenv("DTWPIPE");

   printf("Input parameters\n");
   printf("Parameter 1 = %s\n", envparameter1);
   printf("Parameter 2 = %s\n\n", envparameter2);

/*****************************************************************************/
/*                                                                           */
/*   Open the pipe using the name passed in the DTWPIPE environment          */
/*   variable. This pipe is used to pass back output to the caller via       */
/*   environment variables.                                                  */
/*                                                                           */
/*****************************************************************************/
   if ((pipeHandle = open(pipeName,O_WRONLY)) != NULL) {
      outmsg[0] = '\0';
 
      /***************************************************************/
      /*  Set parameter2 equal to the value of parameter1 and write  */
      /*   to the buffer.                                            */ 
      /***************************************************************/
      strcat(outmsg, "parameter2=\"");
      strcat(outmsg, envparameter1);
      strcat(outmsg, "\"\n");
      /***************************************************************/
      /*   Set the number of rows and columns in the table.          */
      /***************************************************************/
      strcat(outmsg, "table_ROWS=\"2\"\n");
      strcat(outmsg, "table_COLS=\"3\"\n");
 
      /***************************************************************/
      /*   Set the column headers for the table.                     */
      /***************************************************************/
      strcat(outmsg, "table_N_1=\"COL1\"\n");
      strcat(outmsg, "table_N_2=\"COL2\"\n");
      strcat(outmsg, "table_N_3=\"COL3\"\n");

      /***************************************************************/
      /*   Set the arguments in the first row of the table.          */
      /***************************************************************/
      strcat(outmsg, "table_V_1_1=\"");
      strcat(outmsg, argv[1]);
      strcat(outmsg, "\"\n");
      strcat(outmsg, "table_V_1_2=\"");
      strcat(outmsg, argv[2]);
      strcat(outmsg, "\"\n");
      strcat(outmsg, "table_V_1_3=\"");
      strcat(outmsg, argv[3]);
      strcat(outmsg, "\"\n");

      /***************************************************************/
      /*   Set the parameters in the second row of the table.        */
      /***************************************************************/
      strcat(outmsg, "table_V_2_1=\"");
      strcat(outmsg, envparameter1);
      strcat(outmsg, "\"\n");
      strcat(outmsg, "table_V_2_2=\"");
      strcat(outmsg, envparameter2);
      strcat(outmsg, "\"\n");
   
      /***************************************************************/
      /*   Write buffer to pipe                                      */
      /***************************************************************/
      int count=write(pipeHandle,
                      outmsg,
                      strlen(outmsg));
      if (errno) {
         printf("ssyse002.cpp:  write failed %d\n",errno);
         rc = 1;
      }
      else if (count != strlen(outmsg) || errno) {
         printf("ssyse002.cpp:  write failed (%d&#60;&#62;%d)\n",
               count,
               strlen(outmsg));
         rc = 1;
      }
 
      /***************************************************************/
      /*   Close the pipe.                                           */
      /***************************************************************/
      close(pipeHandle);

      if (!rc)
         printf("<em>... ssyse002.exe completes ...</em>\n\n");
 
   }
   else {
      printf("ssyse002.cpp:  open failed %d\n",errno);
   }
  
   return rc;
}
/* end of program : ssyse002.cpp */


