//=======================================================================
// File   : LineChart.java    
// Author : Jian Liang (jian.liang@ebay.sun.com)
// Date   : 1/30/96
// Version: 1.0 ( for Beta2 JDK API)
//
// All Right Reserved. 
//
// Sub-class of AllChart, generating line chart
//
//=======================================================================

import java.awt.*;
import java.applet.*;

public class LineChart extends AllChart2 {
    
    //------------------------------------------------------------------------
    // METHOD: calculateMargin
    //
    //      calculate chart margin size.
    //
    // PARAMETERS: none.
    //
    // RETURN:  none.
    //------------------------------------------------------------------------
    protected  void calculateMargin()
    {
       int i,c;

        m_MarginCx = 30;
        for (c=0; c<m_NumofCols; c++) 
        {
          for (i=0; i<m_NumofPoints; i++) {
                  m_MarginCx = Math.max(m_MarginCx,
                           m_yLabelMetrics.stringWidth(Float.toString(m_YValues[c][i])+5));
          }
        }
        m_MarginCy = m_LabelMetrics.getHeight()+5;
    } 

    
    //------------------------------------------------------------------------
    // METHOD: drawChart
    //
    //      the actual painting.
    //
    // PARAMETERS: g the Graphics object to be drawn on
    //
    // RETURN:  none.
    //------------------------------------------------------------------------   
    public  void drawChart(Graphics g) {

      // draw border
      g.setColor(Color.black); 
      g.drawRect(0,0,size().width-1,size().height-1);

      // draw title
      g.setColor(Color.black);
      drawChartTitle(g);


      // calculate scale
      float xScale  = (float)0.0;
      float yScale  = (float)0.0;
      float minValue = (float)0.0;
      float maxValue = (float)(-1E20);
      int   maxLabelWidth = 5;
      int   zeroLine ; 
      int i,c;

      for (c=0; c<m_NumofCols; c++)
      {
       for ( i = 0; i< m_NumofPoints; i++ ) {
        minValue = Math.min( m_YValues[c][i], minValue );
        maxValue = Math.max( m_YValues[c][i], maxValue );
        if (c==0)
           maxLabelWidth = Math.max(m_LabelMetrics.stringWidth(m_XLabels[i]),maxLabelWidth);
       }
      }

      xScale = (float)(size().width-m_MarginCx + 30 ) / 
               (float)( maxLabelWidth * m_NumofPoints);
      yScale = (float)(size().height - m_MarginCy - m_TitleCy ) / 
               (maxValue-minValue);                      

      zeroLine = size().height-m_MarginCy+(int)(minValue*yScale); 
      
      // draw y axis
      g.setColor(Color.black);
      g.drawLine(m_MarginCx, m_TitleCy, m_MarginCx, size().height - m_MarginCy ) ;

      // draw the chart      
     for (c=0; c<m_NumofCols; c++)
     {
      for ( i = 0; i< m_NumofPoints; i++ ) 
      {                      
          int x0,y0, x1, y1 ;
          
          x1 = (int)(m_MarginCx + i*maxLabelWidth*xScale) ;
          y1 = (int)( zeroLine - m_YValues[c][i]*yScale);
          if ( i > 0) { 
            x0 = (int)(m_MarginCx + (i-1)*maxLabelWidth*xScale) ;
            y0 = (int)( zeroLine - m_YValues[c][i-1]*yScale);

            /*g.setColor( Color.red );*/
            g.setColor(getBrightColor(c));

            g.drawLine( x0 , y0 , x1, y1 ); 
            g.drawLine( x0 , y0+1 , x1, y1+1 ); 
            // draw x axis
            g.setColor(Color.black);
            g.drawLine( x0 , zeroLine, x1, zeroLine);
          }

          // draw x labels,y values,level mark
          g.setColor(Color.black);
          g.setFont(m_LabelFont);
          g.drawString(m_XLabels[i], x1, zeroLine + m_LabelMetrics.getHeight() );
          g.drawLine(x1, zeroLine-2, x1, zeroLine+2 );

          g.drawString(Float.toString(m_YValues[c][i]),2,y1);
          g.drawLine(m_MarginCx-2,y1,m_MarginCx+2,y1);
       }
      }

      //paint big  points
     for (c=0; c<m_NumofCols; c++)
     {
      for ( i = 0; i< m_NumofPoints; i++ ) 
      {                      
          int x1, y1 ;
          x1 = (int)(m_MarginCx + i*maxLabelWidth*xScale) ;
          y1 = (int)( zeroLine - m_YValues[c][i]*yScale);

          /*g.setColor(Color.green.darker());*/
          g.setColor(getBrightColor(c));
          g.fillRect(x1-2,y1-2,4,4);
          g.setColor(Color.black);
          g.drawRect(x1-2,y1-2,4,4);
      }
     }
   } //drawChart
     
} // end of LineChart

