//=======================================================================
// File   : PieChart.java    
// Author : Jian Liang (jian.liang@ebay.sun.com)
// Date   : 1/30/96
// Version: 1.0 ( for Beta2 JDK API)
//
// All Right Reserved. 
//
// Sub-class of AllChart, generating pie chart
//
//=======================================================================

import java.awt.*;
import java.applet.*;

public class PieChart extends AllChart2 {

    //---Declare local variable----------------------------------------------
    double  m_TotalY = 0.0;
    boolean m_FoundNegative = false;
    int     m_ArcAngle[] ;
    float   m_YPercentage[];
    
    //------------------------------------------------------------------------
    // METHOD: calculateMargin
    //
    //      calculate chart margin size.
    //
    // PARAMETERS: none.
    //
    // RETURN:  none.
    //------------------------------------------------------------------------
    protected void calculateMargin(){
          int i;

          m_MarginCx = 120;
          for ( i=0; i < m_NumofPoints; i++) {
             m_MarginCx = Math.max ( m_MarginCx,
                 m_LabelMetrics.stringWidth( (String)m_XLabels[i]) +20);
          } // end of for loop
          m_MarginCx = Math.max( m_MarginCx,
                 m_LabelMetrics.stringWidth("detail information.")) ;
          m_MarginCy = 5;
    } // end of calculateMargin


    //------------------------------------------------------------------------
    // METHOD: drawChart
    //
    //      the actual painting.
    //
    // PARAMETERS: g the Graphics object to be drawn on
    //
    // RETURN:  none.
    //------------------------------------------------------------------------
    public void drawChart(Graphics g) {

          int i;
          float width = size().width - m_MarginCx -10;
          float height = size().height - m_TitleCy - m_MarginCy;
          float arc_width = Math.min(width,height);

          // draw boarder
          g.setColor(Color.black);
          g.drawRect(0,0,size().width-1,size().height-1);

          if (m_FoundNegative == true) {
            g.setColor(Color.red);
            g.setFont(m_LabelFont);
            g.drawString ("Pie Chart not available for negative value.",
                     m_MarginCx + 5, m_TitleCy +5 );
            return;
          } // end of if

          if (m_NumofPoints > 360) {
            g.setColor(Color.red);
            g.setFont(m_LabelFont);
            g.drawString ("Too many data.",
                     m_MarginCx + 5 ,m_TitleCy +5 );
            return;
          } // end of if

          // draw title
          g.setColor(Color.black);
          drawChartTitle(g);

          int pre_angle = 0;

          for (i = 0; i < m_NumofPoints; i++)
          {
              g.setColor(getBrightColor(i));
              g.fillArc ((int)(m_MarginCx+(width/2-arc_width/2)),
                        (int)(m_TitleCy+(height/2-arc_width/2)),
                        (int)arc_width,(int)arc_width,
                         pre_angle,m_ArcAngle[i]);
              pre_angle += m_ArcAngle[i];
          } 

          pre_angle = 0;
          for (i = 0; i < m_NumofPoints; i++) {      
              g.setColor(Color.black);
              g.drawArc ((int)(m_MarginCx+(width/2-arc_width/2)),
                         (int)(m_TitleCy+(height/2-arc_width/2)),
                         (int)arc_width,(int)arc_width,
                         pre_angle,m_ArcAngle[i]);
       
              double r_angle = (pre_angle + m_ArcAngle[i])*Math.PI/180 ;
              int x1 = (int)(m_MarginCx + width/2.0 + 
                             arc_width/2*Math.cos(r_angle)+0.5);
              int y1 = (int)(m_TitleCy + height/2.0 - 
                              arc_width/2*Math.sin(r_angle)+0.5);
              g.drawLine ((int)(m_MarginCx+width/2.0),
                            (int)(m_TitleCy+height/2.0),
                             x1, y1 );

              pre_angle += m_ArcAngle[i];              
              printLegend(g,i);
          } 

     } // end of paint

    //-------------------------------------------------------------------
    // METHOD: AdjustAngle
    // 
    //      adjust desimal lost during the calculation
    //
    // PARAMETERS: none.
    //
    // RETURN:  none.
    //-------------------------------------------------------------------
    public  void AdjustAngle() {

          int i,max_index=0;
          int circle = 0;
          float max_y = 0;
         
          m_ArcAngle = new int[m_NumofPoints];
          for (i=0; i < m_NumofPoints; i++) {
              m_TotalY = m_TotalY + m_YValues[0][i];
              if (m_YValues[0][i] > max_y) {
                   max_y = m_YValues[0][i];
                   max_index = i;
              }
              if (m_YValues[0][i] < 0){
                 m_FoundNegative = true;
                 break;
              }  

          } // end of for loop
      
          m_YPercentage = new float[m_NumofPoints];
          for (i=0; i < m_NumofPoints; i++) {
             m_ArcAngle[i] = (int)(Math.round(360.0*(m_YValues[0][i]/m_TotalY)));
             m_YPercentage[i] = (float)(((int)(m_YValues[0][i]/m_TotalY*1000))/10.0);
             circle += m_ArcAngle[i];
          }
        
          if (circle != 360) {
             circle = 360 - circle;
             m_ArcAngle[max_index] += circle;
          }

    }//end of AdjustAngle

    //-------------------------------------------------------------------
    // METHOD: initChart 
    //
    //     overwrite AllChart.initChart, adjust angles before paint. 
    //
    // PARAMETERS: dataSource -- an URL or ChartServer ID.
    //             connectionType -- 0 for URL, 1 for ChartServer     
    //
    // RETURN:  none.
    //-------------------------------------------------------------------
    public void initChart(int depth,int radius, int numRow,int numCol, float[][] values,String ChartTitle)
    {
           boolean rtn_val = true;
           super.initChart(depth,radius,numRow,numCol,values,ChartTitle);
           if (rtn_val == true) {
              AdjustAngle();
           }
     }//end of initChart


    //-------------------------------------------------------------------
    // METHOD: printLegend 
    //
    //     print legend according to different cursor position 
    //
    // PARAMETERS:  g -- the Graphics to draw on
    //              i -- m_YValues index reference 
    //
    // RETURN:  none.
    //-------------------------------------------------------------------
    public void printLegend(Graphics g, int i) {

         int w = m_LabelMetrics.stringWidth(m_XLabels[i]); 
         int h = m_LabelMetrics.getHeight(); 

          g.setColor(getBrightColor(i));
          g.fillRect(3,(i-1)*h+m_TitleCy+4,10,h-5);

          // print info
          g.setColor(Color.black);
          g.drawRect(2,(i-1)*h+m_TitleCy+3,9,h-4);
      
          g.setFont(m_LabelFont);
          g.drawString(m_XLabels[i]+", "+Float.toString(m_YValues[0][i])+
                        " ("+Float.toString(m_YPercentage[i])+"%)"
                        ,25,
                        (int)(m_TitleCy+h*i));

    }// end of printLegend



} // end of PieChart

