//=======================================================================
// File   : ThreeDHBarChart.java    
// Author : Jian Liang (jian.liang@ebay.sun.com)
// Date   : 1/30/96
// Version: 1.0 ( for Beta2 JDK API)
//
// All Right Reserved. 
//
// Sub-class of AllChart, generating 3D horizontal bar chart
//
//=======================================================================

import java.awt.*;

public class ThreeDHBarChart extends AllChart2 {
        
    //------------------------------------------------------------------------
    // METHOD: calculateMargin
    //
    //      calculate chart margin size.
    //
    // PARAMETERS: none.
    //
    // RETURN:  none.
    //------------------------------------------------------------------------
    protected  void calculateMargin()
    {     
         int i;

         m_MarginCy = 30;
         m_MarginCx = 0;
         for ( i =0; i < m_NumofPoints; i++ ) {
                   m_MarginCx = Math.max( m_MarginCx ,
                    m_LabelMetrics.stringWidth( (String)m_XLabels[i])+ 10 ) ;
         }
    }
    

    //------------------------------------------------------------------------
    // METHOD: drawChart
    //
    //      the actual painting.
    //
    // PARAMETERS: g the Graphics object to be drawn on
    //
    // RETURN:  none.
    //------------------------------------------------------------------------
    public  void drawChart(Graphics g) 
    {       
      float xScale  = (float)0.0;
      float yScale  = (float)0.0;
      float minValue = (float)0.0;
      float maxValue = (float)(-1E20);
      int   max_index = 0;
      int   zeroLine ; 
      int   barSize ;
      int i;
      
      // draw border
      g.setColor(Color.black);
      g.drawRect(0,0,size().width-1,size().height-1);

      // Title 
      g.setColor(Color.black);
      drawChartTitle(g);
      
      if (m_NumofPoints == 0)
         return;

      // calculate scale
      for ( i = 0; i< m_NumofPoints; i++ ) {
        minValue = Math.min( m_YValues[0][i], minValue );
        if (m_YValues[0][i] > maxValue) {
          maxValue = m_YValues[0][i];
          max_index = i;
        }
      }
      xScale = (float)(size().width-m_MarginCx-30) / 
               (maxValue-minValue);
      yScale = (float)(size().height - m_MarginCy - m_TitleCy ) / 
               (float)(m_NumofPoints * 1.5 * m_LabelMetrics.getHeight());            

      zeroLine = m_MarginCx-(int)(minValue*xScale); 
      barSize = (int)(m_LabelMetrics.getHeight() * yScale) ;
      
      // draw y axis
      g.setColor(Color.black);
      g.drawLine(zeroLine, m_TitleCy, zeroLine, size().height - m_MarginCy );
    
      int start = zeroLine;

      // draw the chart      
      for ( i = 0; i< m_NumofPoints; i++ ) 
      {    
          g.setColor(Color.black);
          g.setFont(m_LabelFont);
          g.drawString(m_XLabels[i], 
                       2,
                       (int)(m_TitleCy+i*barSize*1.5) +
                       m_LabelMetrics.getHeight() ); 

          int x0, y0, w0 ;
          x0 = (m_YValues[0][i] >= 0) ? zeroLine : m_MarginCx ;
          y0 = (int)(m_TitleCy + barSize*1.5*i);
          w0 = (int)(Math.abs(m_YValues[0][i]) * xScale  );

          if (m_YValues[0][i] >= 0 ) { 
             g.setColor( getBrightColor(i).darker() );
             g.fillRect( x0 , y0 , w0+5, barSize ); 

             g.setColor(Color.black);
             g.drawRect( x0 , y0 , w0+5, barSize ); 

             g.setColor( getBrightColor(i) );
             g.fillRect( x0 , y0+5 , w0, barSize );
 
             g.setColor( getBrightColor(i).darker() );
             int top_x[];
             int top_y[];
             top_x = new int[4];
             top_y = new int[4];
             top_x[0] = x0+w0;
             top_x[1] = x0+w0+5;
             top_x[2] = x0+5+w0;
             top_x[3] = x0+w0;
             top_y[0] = y0+5;
             top_y[1] = y0;
             top_y[2] = y0+barSize;
             top_y[3] = y0+5+barSize;
             g.fillPolygon(top_x,top_y,4); 

             g.setColor(Color.black);
             g.drawRect( x0 , y0+5 , w0, barSize );

             g.drawLine( x0+w0+5,y0+barSize,x0+w0,y0+5+barSize);
             g.drawLine( x0+w0+5,y0,x0+w0,y0+5);
          }
          else { // for negative values
             g.setColor( getBrightColor(i).darker() );
             g.fillRect( x0-5 , y0 , w0+5, barSize );
 
             g.setColor(Color.black);
             g.drawRect( x0-5 , y0 , w0+5, barSize );
 
             g.setColor( getBrightColor(i) );
             g.fillRect( x0 , y0+5 , w0, barSize );
 
             g.setColor( getBrightColor(i).darker() );
             int top_x[];
             int top_y[];
             top_x = new int[4];
             top_y = new int[4];
             top_x[0] = x0; 
             top_x[1] = x0-5;
             top_x[2] = x0-5;
             top_x[3] = x0; 
             top_y[0] = y0+5;
             top_y[1] = y0; 
             top_y[2] = y0+barSize; 
             top_y[3] = y0+5+barSize;
             g.fillPolygon(top_x,top_y,4);
 
             g.setColor(Color.black);
             g.drawRect( x0 , y0+5 , w0, barSize );
 
             g.drawLine( x0-5,y0+barSize,x0,y0+5+barSize);
             g.drawLine( x0-5,y0,x0,y0+5);
  
          }

          // draw legend
          if (m_YValues[0][i] < 0 ) {
            start = m_MarginCx;
            g.drawLine(start,size().height-m_MarginCy-2,start,size().height-m_MarginCy+2);
            g.drawString(Float.toString(m_YValues[0][i]),start,size().height-m_MarginCy+10);
          }
          else {
            g.drawLine(w0+x0,size().height-m_MarginCy-2,w0+x0,size().height-m_MarginCy+2);
            g.drawString(Float.toString(m_YValues[0][i]),w0+x0,size().height-m_MarginCy+10);
          }

      }

      // draw x axis
      g.setColor(Color.black);
      int tmp = (int)(Math.abs(m_YValues[0][max_index]*xScale));
      g.drawLine(start,size().height - m_MarginCy,zeroLine+tmp,size().height - m_MarginCy);

    }    

}//end of ThreeDHBarChart   

