//=======================================================================
// File   : VBarChart.java    
// Author : Jian Liang (jian.liang@ebay.sun.com)
// Date   : 1/30/96
// Version: 1.0 ( for Beta2 JDK API)
//
// All Right Reserved. 
//
// Sub-class of AllChart, generating vertical bar chart
//
//=======================================================================

import java.awt.*;

public class VBarChart extends AllChart2 {


    //------------------------------------------------------------------------
    // METHOD: calculateMargin
    //
    //      calculate chart margin size.
    //
    // PARAMETERS: none.
    //
    // RETURN:  none.
    //------------------------------------------------------------------------
    protected  void calculateMargin(){     
         int i;
         
         m_MarginCx = 5;
         m_MarginCy = m_LabelMetrics.getHeight() + 5;     
    }


    //------------------------------------------------------------------------
    // METHOD: drawChart
    //
    //      the actual painting.
    //
    // PARAMETERS: g the Graphics object to be drawn on
    //
    // RETURN:  none.
    //------------------------------------------------------------------------
    public  void drawChart(Graphics g) 
    {       
      float xScale  = (float)0.0;
      float yScale  = (float)0.0;
      float minValue = (float)0.0;
      float maxValue = (float)(-1E20);
      int   maxLabelWidth = 0;
      int   zeroLine ; 
      int   barSize = 0 ;
      int i;

      // draw border
      g.setColor(Color.black); 
      g.drawRect(0,0,size().width-1,size().height-1);

      // Title
      g.setColor(Color.black);
      drawChartTitle(g);

      // calculate scale
      for ( i = 0; i< m_NumofPoints; i++ ) {
        minValue = Math.min( m_YValues[0][i], minValue );
        maxValue = Math.max( m_YValues[0][i], maxValue );
        maxLabelWidth = Math.max(m_LabelMetrics.stringWidth(m_XLabels[i]),
                        maxLabelWidth);
      }

      xScale = (float)(size().width-m_MarginCx) / 
               (float)( maxLabelWidth * m_NumofPoints * 2 );
      yScale = (float)(size().height - m_MarginCy - m_TitleCy ) / 
               (maxValue-minValue);                      

      zeroLine = size().height-m_MarginCy+(int)(minValue*yScale); 
      barSize = (int)( maxLabelWidth * xScale)  ;
      
      // draw x axis
      g.setColor(Color.black);
      g.drawLine(m_MarginCx, zeroLine, size().width-10, zeroLine ) ;

      // draw the chart      
      for ( i = 0; i< m_NumofPoints; i++ ) {
          
          int x0,y0, h0 ;
          x0 = (int)(m_MarginCx + i*2*barSize) ;
          y0 = (m_YValues[0][i] < 0) ? zeroLine:(int)( zeroLine - m_YValues[0][i]*yScale);
          h0 = (int)(Math.abs(m_YValues[0][i])*yScale );

          g.setColor( getBrightColor(i) );
          g.fillRect( x0 , y0 , barSize, h0 ); 

          g.setColor(Color.black);
          g.drawRect( x0 , y0 , barSize, h0 ); 

          // print y values on screen
          g.setColor(Color.black);
          g.setFont(m_LabelFont);
          g.drawString(m_XLabels[i], 
                       (int)(m_MarginCx + i*2*barSize) ,
                       zeroLine + m_LabelMetrics.getHeight() );

          if (m_YValues[0][i] >= 0) {
             g.drawString(Float.toString(m_YValues[0][i]),x0+(int)(barSize/2),y0);
          }
          else {
             g.drawString(Float.toString(m_YValues[0][i]),x0+(int)(barSize/2),y0+h0+10);
          }
      }
    }    
}

